using System.Net.Http.Headers;
using System.Data;
using System.Runtime.CompilerServices;
using WingServerCommon.Service;
using WingInterfaceLibrary.Interface;
using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.DTO.Education;
using WingInterfaceLibrary.Request.Education;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Interfaces.Cache;
using WingServerCommon.Utilities;
using WingInterfaceLibrary.Result;
using System.Linq;
using WingServerCommon.Config;
using WingServerCommon.Config.Parameters;
using WingServerCommon.Log;
using System;
using WingInterfaceLibrary.Request.Lock;
using WingInterfaceLibrary.Internal.Interface;
using WingInterfaceLibrary.Rtc;
using WingEducationService.Common;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Request.Management;
using WingInterfaceLibrary.DB.Request;
using System.Timers;

namespace WingEducationService.Service
{
    /// <summary>
    /// 教培服务
    /// </summary>
    public partial class EducationService : JsonRpcService, IEducationService
    {
        private ICourseDBService _courseDBService;
        private IUserDBService _userDBService;
        private IAuthenticationService _authenticationService;
        private IManagementService _managementService;
        private IEducationDBService _educationDBService;
        private LockManager _lockManager;
        private IWingRtcService _wingRtcService;
        private ILockService _lockService;
        private IDeviceInfoDBService _deviceDBService;
        private ILiveRoomDBService _liveRoomDBService;
        private IMasterInteractionCenterService _masterInteractionCenterService;
        private LiveHeartRateManager _liveHeartRateManager;
        private VisitorLiveHeartRateManager _visitorLiveHeartRateManager;
        private IPaymentDBService _paymentDBService;
        private IPaymentService _paymentService;
        private INotificationService _notificationService;
        private IAnnouncementDBService _announcementDBService;
        private IRtcService _rtcService;
        private string _otherNodes = EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "OtherNodes").Value;        
        private string _curCDNHost => EnvironmentConfigManager.GetParammeter<StringParameter>("Storage", "CDNServerUrl").Value;   
        private string _vodHostNode => EnvironmentConfigManager.GetParammeter<StringParameter>("VodPlayer", "VodHostNode").Value;

        
        //计时器
        private DateTime _scheduledTime;
        //线程定时器
        private Timer _timerCallPy = null;
        //执行周期
        private int _intervalMinutes = 5;
        //默认过期时间
        private int _defaultExpiredDay = 1;
        /// <summary>
        /// 默认构造
        /// </summary>
        public EducationService()
        {
            _liveHeartRateManager = new LiveHeartRateManager(HandleClientJoined, HandleClientNetworkErr, HandleClientLeave);
            _visitorLiveHeartRateManager = new VisitorLiveHeartRateManager(VisitorHandleClientJoined, VisitorHandleClientNetworkErr, VisitorHandleClientLeave);
        }

        /// <summary>
        /// 初始化服务
        /// </summary>
        public override void Load(JsonRpcClientPool jsonRpcClientPool)
        {
            base.Load(jsonRpcClientPool);
            _courseDBService = GetProxy<ICourseDBService>();
            _userDBService = GetProxy<IUserDBService>();
            _liveRoomDBService = GetProxy<ILiveRoomDBService>();
            _educationDBService = GetProxy<IEducationDBService>();
            _authenticationService = GetProxy<IAuthenticationService>();
            _wingRtcService = GetProxy<IWingRtcService>();
            _rtcService = GetProxy<IRtcService>();
            _deviceDBService = GetProxy<IDeviceInfoDBService>();            
            _masterInteractionCenterService = GetProxy<IMasterInteractionCenterService>();
            _lockService = GetProxy<ILockService>();
            _paymentDBService = GetProxy<IPaymentDBService>();
            _paymentService = GetProxy<IPaymentService>();
            _notificationService = GetProxy<INotificationService>();
            _managementService = GetProxy<IManagementService>();
            _announcementDBService = GetProxy<IAnnouncementDBService>();
            var inProcessServiceSettings = ConfigurationManager.GetParammeter<StringParameter>("Services", "InProcess").Value;
            var remoteServiceSettings = ConfigurationManager.GetParammeter<StringParameter>("Services", "Remote").Value;
            if (!inProcessServiceSettings.Contains("LockService") && !remoteServiceSettings.Contains("LockService"))
            {
                _lockManager = new LockManager();
            }
            else
            {
                _lockManager = new LockManager(ApplyLock, ReleaseLock);
            }
        }

        #region Basic Method

        /// <summary>
        /// 通用异常处理方法
        /// </summary>
        /// <param name="customerRpcCodeEnum">异常标识码</param>
        /// <param name="msg">异常信息</param>
        /// <param name="internalMessage">内部异常信息，非必填</param>
        private void ThrowCustomerException(CustomerRpcCode customerRpcCodeEnum, string msg, string internalMessage = null)
        {
            ThrowRpcException((int)customerRpcCodeEnum, msg, internalMessage);
        }

        /// <summary>
        /// Apply Lock
        /// </summary>
        /// <param name="lockKey">key</param>
        /// <returns>是否锁定</returns>
        private bool ApplyLock(string lockKey)
        {
            try
            {
                var result = _lockService.ApplyLockAsync(new ApplyLockRequest
                {
                    LockKey = lockKey
                }).Result;
                return result.IsSuccess;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveCourseService ApplyLock err, lock key:{lockKey}, ex:{ex}");
                return false;
            }
        }

        /// <summary>
        /// Release Lock
        /// </summary>
        /// <param name="lockKey">key</param>
        /// <returns></returns>
        private bool ReleaseLock(string lockUniqueCode)
        {
            try
            {
                var result = _lockService.ReleaseLockAsync(new ReleaseLockRequest
                {
                    LockUniqueCode = lockUniqueCode
                }).Result;
                return result.IsSuccess;
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"LiveCourseService ReleaseLock err, release key:{lockUniqueCode}, ex:{ex}");
                return false;
            }
        }

        /// <summary>
        /// 自定异常处理方法
        /// </summary>
        private void ThrowRpcException(CustomerRpcCode code, string message, string internalMessage = null)
        {
            base.ThrowRpcException((int)code, message, internalMessage ?? message);
        }

        /// <summary>
        /// 服务启动后注册执行
        /// </summary>
        public override async void LoadAfterRegister()
        {
            //todo 启动后判断是否有要开始直播的提醒
            CaculateScheduleSet();
            //todo 加载在线游客到内存中
            InitOnlineVisitorLiveHeartRate();
        }
        #endregion

        /// <summary>
        /// 获取标签列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>标签信息</returns>
        /// <errorCodes></errorCodes>
        public async Task<IList<CourseLabelDTO>> GetCourseLabelsAsync(QueryCourseLabelListRequest request)
        {
            var result = await _educationDBService.GetCourseLabelsDBAsync(request, true);
            return result;
        }

        /// <summary>
        /// 获取Token
        /// </summary>
        /// <param name="token"></param>
        /// <returns>用户id</returns>
        private async Task<string> GetClientIdByTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result.ClientId; //return User code 
        }

        /// <summary>
        /// 获取Token
        /// </summary>
        /// <param name="token"></param>
        /// <returns>用户id</returns>
        private async Task<TokenDTO> GetClientInfoByTokenAsync(string token)
        {
            var request = new TokenRequest() { Token = token };
            var result = await _authenticationService.GetTokenAsync(request);
            return result;
        }

        /// <summary>
        /// 客户端根据创建人获取创建的班级
        /// </summary> 
        /// <param name="request">创建人code</param>
        /// <returns></returns>
        public async Task<PageResult<UserGroupDTO>> GetUserGroupListsByCreatorCodeAsync(QueryUserGroupPageRequest request)
        {
            var clientId = await GetClientIdByTokenAsync(request.Token);
            if (string.IsNullOrEmpty(clientId))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCreatorCodeIsEmpty, "UserGroup creator code is empty");
            }
            if (string.IsNullOrEmpty(request.TeacherCode))
            {
                request.TeacherCode = clientId;
            }
            var req = new QueryUserGroupPageDBRequest()
            {
                Keyword = request.Keyword,
                TeacherCode = request.TeacherCode,
                PageIndex = request.PageIndex,
                PageSize = request.PageSize
            };
            var result = await _educationDBService.FindUserGroupsPagesDBAsync(req);
            return result;
        }

        /// <summary>
        /// 查询所有学员列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<PageResult<UserExtendDTO>> GetAllStudentsAsync(GetUserPagesRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var excludeUserCodes = request.ExcludeUserCodes ?? new List<string>();
            excludeUserCodes.Add(userCode);
            if (string.IsNullOrEmpty(userCode))
            {
                ThrowRpcException(CustomerRpcCode.UserNotFind, "User not find");
            }
            var filter = new Dictionary<string, string>();
            if (!string.IsNullOrWhiteSpace(request.QueryState))
            {
                filter.Add("UserState", request.QueryState);
            }
            filter.Add("UserName", request.Keyword);
            filter.Add("Phone", request.Keyword);
            filter.Add("Email", request.Keyword);
            filter.Add("FullName", request.Keyword);
            
            var findUserPageByOrgsRequest = new FindUserPageByOrgsRequest
            {
                CurrentPage = request.PageIndex,
                PageSize = request.PageSize,
                Filter = filter,
                IsFuzzy = true,
                ExcludeUserCodes = excludeUserCodes
            };
            var list = new PageResult<UserExtendDTO>();
            var userListResult = await _userDBService.FindUserPagesByOrgsAsync(findUserPageByOrgsRequest);
            if (userListResult?.PageData?.Count > 0)
            {
                var dataList = new List<UserExtendDTO>();
                foreach(var item in userListResult.PageData)
                {
                    var model = new UserExtendDTO()
                    {
                        UserCode = item.UserCode,
                        UserName = item.UserName,
                        FullName = item.FullName,
                        HeadImageUrl = item.HeadImageUrl,         
                        Language = item.Language,
                        Signature = item.Signature,
                        FieldList = item.FieldList,
                        IsDirector = item.IsDirector,
                        PositionName = item.PositionName,
                        RankName = item.RankName,
                        ApplyState = item.ApplyState,
                        PositionCodes = item.PositionCodes,
                        Phone = item.Phone,
                        Email = item.Email,
                        NickName = item.NickName,
                        OrganizationCode = item.OrganizationCode,
                        OrganizationName = item.OrganizationName,
                        RootOrganizationCode = item.RootOrganizationCode,
                        BindEmergencyExpertCode = item.BindEmergencyExpertCode,
                        RootOrganizationName = item.RootOrganizationName,
                        BindDevices = item.BindDevices,
                        LastIP = item.LastIP,
                        Logintimes = item.Logintimes,
                        UserState = item.UserState,
                    };
                    dataList.Add(model);
                }
                list.PageData = dataList;
                list.TotalCount = userListResult.DataCount;
                list.PageIndex = userListResult.CurrentPage;
                list.PageSize = userListResult.PageSize;
            }
            return list;
        }

        /// <summary>
        /// 新增用户组
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> AddUserGroupAsync(UserGroupRequest request)
        {
            if (string.IsNullOrEmpty(request.Name))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupNameIsEmpty, "UserGroup name is empty");
            }
            if (request.MaxPeople <= 0)
            {
                ThrowRpcException(CustomerRpcCode.UserGroupMaxPeopleNotValid, "UserGroup max people not valid");
            }
            //用户权限验证
            if (string.IsNullOrEmpty(request.CreatorCode))
            {
                var tokenInfo = await GetClientInfoByTokenAsync(request.Token);
                if (tokenInfo == null || string.IsNullOrEmpty(tokenInfo.ClientId))
                {
                    ThrowRpcException(CustomerRpcCode.UserGroupCreatorCodeIsEmpty, "UserGroup creator code is empty");
                }
                request.CreatorCode = tokenInfo.ClientId;
            }
            else
            {
                //var userInfo = CacheMaintenance.Instance.Get<IUserInfoManager>().Where(x => x.Code == request.CreatorCode)?.FirstOrDefault();
                var userInfo = await _userDBService.FindUserDTOByCodeAsync(request.CreatorCode, false);
                if (string.IsNullOrEmpty(userInfo?.UserCode))
                {
                    ThrowRpcException(CustomerRpcCode.UserGroupCreatorCodeIsEmpty, "UserGroup creator code is empty");
                }
            }
            var result = await _educationDBService.AddUserGroupDBAsync(request);
            return result;
        }

        // /// <summary>
        // /// 查询用户分页
        // /// </summary>
        // /// <param name="request"></param>
        // /// <returns></returns>
        // public async Task<PageResult<StudentInfoDTO>> FindUserGroupStudentsPagesAsync(QueryStudentByGroupCodePageRequest request)
        // {
        //     if (string.IsNullOrEmpty(request.UserGroupCode))
        //     {
        //         ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroup code is empty");
        //     }
        //     var list = new PageResult<StudentInfoDTO>();
        //     var dataInfo = await _educationDBService.FindUserGroupsByGroupCodeDBAsync(request.UserGroupCode, request.Keyword);
        //     if (dataInfo != null && dataInfo.Students?.Count > 0)
        //     {
        //         var studentList = dataInfo.Students;
        //         if (studentList?.Count > 0)
        //         {
        //             var filterList = studentList.Skip(request.PageSize * (request.PageIndex - 1))?.Take(request.PageSize)?.ToList();
        //             list.PageData = filterList;
        //             list.TotalCount = studentList.Count;
        //         }
        //     }
        //     list.PageIndex = request.PageIndex;
        //     list.PageSize = request.PageSize;
        //     return list;
        // }

        /// <summary>
        /// 用户组移除用户
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns></returns>
        public async Task<bool> RemoveStudentsByGroupCodeAsync(UserGroupFilterRequest request)
        {
            if (string.IsNullOrEmpty(request.UserGroupCode))
            {
                ThrowRpcException(CustomerRpcCode.UserGroupCodeIsEmpty, "UserGroup code is empty");
            }
            if (request.StudentCodes == null || request.StudentCodes.Count <= 0)
            {
                ThrowRpcException(CustomerRpcCode.UserGroupStudentsIsEmpty, "UserGroup students is empty");
            }
            var result = await _educationDBService.RemoveStudentsByGroupCodeDBAsync(request.UserGroupCode, request.StudentCodes);
            return result;
        }
    }
}